@@ -4,16 +4,17 @@
     symbolx
     copyright 2022, Carlos Gaete-Morales, DIW-Berlin 
 """
-__version__ = (0, 2, 0)
+__version__ = (0, 3, 0)
 __author__ = 'Carlos Gaete-Morales'
 
 
 from .parsers.parser_csv import symbol_parser_csv, load_csv
 from .parsers.parser_feather import symbol_parser_feather, load_feather
 from .parsers.parser_gdx import symbol_parser_gdx, load_gdx
-from .handler import DataCollection, SymbolsHandler
+from .handler import DataCollection
 from .utils import compress_subdirs, compress_dir, unzip_all, unzip
-from .symbols import Symbol, from_feather
+from .symbols import SymbolsHandler, Symbol, from_feather
+from .settings import settings
 
 
 
