@@ -42,7 +42,7 @@ def symbol_parser_feather(folder: str, symbol_names: list=[], zip_extension=None
             symbol_list.append(symbol_dict)
     return symbol_list
 
-def load_feather(path:str, keep_zeros:bool, zip_extension, **kwargs):
+def load_feather(path:str, zip_extension, **kwargs):
     '''
     Load custom feather file.
     '''
@@ -56,10 +56,10 @@ def load_feather(path:str, keep_zeros:bool, zip_extension, **kwargs):
         target_fpath = path_parts[1]
         with zipfile.ZipFile(zip_fpath, mode="r") as zip_io:
             with zip_io.open(target_fpath) as the_file:
-                arr = ka._from_feather(the_file, keep_zeros=keep_zeros, use_threads=threads)
+                arr = ka._from_feather(the_file, use_threads=threads)
         return arr
     else:
-        return ka._from_feather(path, keep_zeros=keep_zeros, use_threads=threads)
+        return ka._from_feather(path, use_threads=threads)
 
 def _info_feather(path:str, tool_name:str):
     '''
