@@ -58,11 +58,10 @@ def symbol_parser_gdx(folder: str, symbol_names: list=[], zip_extension=None, **
                                         'zip_extension':zip_extension,
                                         'inf_to_zero':True, # included with default value. This can be changed later in handler.py def add_custom_attr()
                                         'verbose':False,
-                                        'keep_zeros':False,
                                         })
     return symbol_list
 
-def load_gdx(symbol_name: str, value_type: str='v', path: str='', gams_dir: str= None, inf_to_zero:bool=True, verbose:bool=False, keep_zeros:bool=False, zip_extension=None, **kwargs):
+def load_gdx(symbol_name: str, value_type: str='v', path: str='', gams_dir: str= None, inf_to_zero:bool=True, verbose:bool=False, zip_extension=None, **kwargs):
     '''
     Load custom GDX file.
 
@@ -104,7 +103,7 @@ def load_gdx(symbol_name: str, value_type: str='v', path: str='', gams_dir: str=
                 value = raw_coo[:, nrdims].astype(float)
                 index = {dim: raw_coo[:,idx] for idx, dim in enumerate(metadata['dims'])}
                 coords = {dim: metadata['coords'][dim] for dim in metadata['dims']}
-                return {'data': (index, value),'coords': coords, 'keep_zeros': keep_zeros}
+                return {'data': (index, value),'coords': coords}
     else:
         metadata = _gdx_get_symbol_data_dict(symbol_name=symbol_name, gdx_file=path, gams_dir=gams_dir)
         symbol = _gdx_get_symbol_array_str(symbol_name=symbol_name, gdx_file=path, gams_dir=gams_dir)
@@ -121,7 +120,7 @@ def load_gdx(symbol_name: str, value_type: str='v', path: str='', gams_dir: str=
         value = raw_coo[:, nrdims].astype(float)
         index = {dim: raw_coo[:,idx] for idx, dim in enumerate(metadata['dims'])}
         coords = {dim: metadata['coords'][dim] for dim in metadata['dims']}
-        return {'data': (index, value),'coords': coords, 'keep_zeros': keep_zeros}
+        return {'data': (index, value),'coords': coords}
 
 def _symbols_list_from_gdx(filename: str = None, gams_dir: str = None):
     """ It returns a list of symbols' names contained in the GDX file
