@@ -1,13 +1,159 @@
 import os
-import json
+import glob
+import uuid
 import itertools
 import numpy as np
 import karray as ka
-from typing import Union
-from .handler import SymbolsHandler, from_feather_dict
+import json
+from json import JSONDecodeError
+from typing import Union, List, Dict
+from .handler import DataCollection
 from .settings import settings, allowed_string, value_type_name_map
 
 
+class SymbolsHandler:
+    def __init__(self, method:str, folder_path:str=None, obj:DataCollection=None):
+        ''' 
+        method: "folder" or "object"
+        kwargs:
+            folder_path: path to folder with symbol files
+            object: DataCollection object
+        '''
+        assert isinstance(method, str), "Arg 'method' must be a string."
+        assert method in ["folder", "object"], "Arg 'method' must be either 'folder' or 'object'"
+        self.method = method
+        self.folder_path = None
+        self.symbols_book = None
+        self.symbol_handler_token = str(uuid.uuid4()) # TODO: this can be changed by hashing the input file
+        self.saved_symbols = {}
+        self.token_info = None
+        self.input_method(method=method, folder_path=folder_path, obj=obj)
+
+    def input_method(self, method:str, **kwargs):
+        if method == "object":
+            self.from_object(obj=kwargs['obj'])
+        elif method == "folder":
+            self.from_folder(folder_path=kwargs['folder_path'])
+        else:
+            raise Exception('A method mus be provided from either "object" or "folder"')
+
+    def from_object(self, obj:DataCollection):
+        self.symbols_book = obj.symbols_book
+        self.collector = obj.collector
+        self.short_names = obj.short_names
+        for n_v in self.symbols_book:
+            settings.append((*n_v,self.symbol_handler_token))
+
+
+    def from_folder(self, folder_path:str=None):
+        self.token_info = {}
+        self.folder_path = folder_path
+        files = glob.glob(os.path.join(self.folder_path, "*.feather"))
+        self.symbols_book = {}
+        for file in files:
+            loaded_file_dict = from_feather_info(file)
+            self.symbols_book[(loaded_file_dict['name'],loaded_file_dict['value_type'])] = file
+
+            token = loaded_file_dict['symbol_handler_token']
+            settings.append((loaded_file_dict['name'],loaded_file_dict['value_type'],loaded_file_dict['symbol_handler_token']))
+
+            if token not in self.token_info:
+                self.token_info[token] = {}
+            self.token_info[token][(loaded_file_dict['name'],loaded_file_dict['value_type'])] = file
+        if len(self.token_info) > 1:
+            print(f"There are Symbols' files with different symbol_handler_token in {folder_path}")
+            print("       Symbol's scenario short names or id's might be in conflict.")
+            print("       Make sure all Symbols are from the same symbol_handler_token")
+            print("       Otherwise, it may happen, for example, different scenarios have the same id.")
+            print("       See 'token_info' attribute of SymbolsHandler for more information")
+
+
+    def append(self, **kwargs):
+        for name in kwargs:
+            symbol = kwargs[name]
+            assert len(set(name).difference(set(allowed_string))) == 0, f"Symbol name '{name}' contains special characters. Please, change the name of the symbol. Allowed chars are: {allowed_string}"
+            symbol.name = name
+            self.saved_symbols[(name, symbol.value_type)] = symbol
+
+    def save(self, folder_path=None, appended_or_all:str='all', gls=None, overwrite:bool=False):
+        files = glob.glob(os.path.join(folder_path,'*.feather'))
+        existing_pairs = {}
+        print("Checking folder for duplicate symbol names...")
+        for ff in files:
+            file = os.path.basename(ff)
+            name_sections = file.split('.')
+            name = name_sections[0]
+            value_type = name_sections[1]
+            token = name_sections[2]
+            assert (name,value_type) not in existing_pairs, f"Only one symbol name {name}.{value_type} should be found in '{folder_path}' folder. Two or more found\nTokens: {existing_pairs[(name,value_type)]} and {token}"
+            existing_pairs[(name,value_type)] = token
+        print("Checking passed")
+        if folder_path is None:
+            assert self.folder_path is not None, "folder_path must be provided"
+            folder_path = self.folder_path
+        os.makedirs(folder_path, exist_ok=True)
+        if appended_or_all == 'all':
+            symbol_map = []
+            assert gls is not None, "if 'appended_or_all' is 'all' then provide gls=globals() to able to capture all variables that are Symbol instances"
+            for name, symbol in gls.items():
+                if isinstance(symbol, Symbol):
+                    symbol.name = name
+                    symbol_map.append(symbol)
+        elif appended_or_all == 'appended':
+            symbol_map = list(self.saved_symbols.values())
+        else:
+            raise Exception(f"Function argument 'appended_or_all' must be 'appended' or 'all'. Given: {appended_or_all}")
+
+        for symbol in symbol_map:
+            if not overwrite:
+                assert (symbol.name,symbol.value_type) not in existing_pairs, f"Only one symbol name {symbol.name}.{symbol.value_type} should be saved in '{folder_path}' folder. Existing file found:\n{symbol.name}.{symbol.value_type}.{existing_pairs[(symbol.name,symbol.value_type)]}.feather"
+            file_name = f"{symbol.name}.{symbol.value_type}.{symbol.symbol_handler_token}.feather"
+            file_path = os.path.join(folder_path,file_name)
+            symbol.to_feather(file_path)
+
+    def get_info(self, symbol_name, value_type):
+        assert (symbol_name, value_type) in self.symbols_book, f"Pairs {(symbol_name, value_type)} not present in the symbols_book attribute of SymbolsHandler"
+        if isinstance(self.symbols_book[(symbol_name, value_type)], dict):
+            return self.symbols_book[(symbol_name, value_type)]
+        elif isinstance(self.symbols_book[(symbol_name, value_type)], str):
+            return self.symbols_book[(symbol_name, value_type)]
+
+    def __repr__(self):
+        return f'''SymbolsHandler(method='{self.method}')'''
+
+def from_feather_dict(path, use_threads=True, with_="polars"):
+    import pyarrow.feather as ft
+    arr = ka.from_feather(path, use_threads=use_threads, with_=with_)
+    restored_table = ft.read_table(path, use_threads=use_threads)
+    custom_meta_key = 'symbolx'
+    restored_meta_json = restored_table.schema.metadata[custom_meta_key.encode()]
+    restored_meta = recurse(restored_meta_json)
+    return dict(array=arr,**restored_meta)
+
+def from_feather_info(path):
+    import pyarrow.feather as ft
+    restored_table = ft.read_table(path)
+    custom_meta_key = 'symbolx'
+    restored_meta_json = restored_table.schema.metadata[custom_meta_key.encode()]
+    restored_meta = json.loads(restored_meta_json)
+    return dict(name=restored_meta['name'], value_type=restored_meta['value_type'], symbol_handler_token=restored_meta['symbol_handler_token'])
+
+def recurse(d):
+    if isinstance(d, dict):
+        loaded_d = d
+    elif isinstance(d, bytes):
+        loaded_d = json.loads(d)
+    else:
+        return d
+    new_dc = {}
+    for k, v in loaded_d.items():
+        if k.isdigit():
+            r = int(k)
+        else:
+            r = k
+        new_dc[r] = recurse(v)
+    return new_dc
+
 def build_array(symbol_name:str, value_type:str, symbol_handler:SymbolsHandler):
     """
     Create an array of all existing scenarios
@@ -28,9 +174,6 @@ def insert_id_dim(symbol_name:str, value_type:str, scenario_id:str, symbol_handl
     narray = oarray.add_dim(id=symbol_handler.short_names[scenario_id])
     return narray
 
-def from_feather(path):
-    return Symbol(**from_feather_dict(path))
-
 
 class Symbol:
     def __init__(
@@ -62,7 +205,7 @@ class Symbol:
             self.symbol_handler_token= symbol_handler_token
         self.check_input()
         # optional attributes
-        self.df = None
+        self.dataframe = None
 
 
     def __setattr__(self, name, value):
@@ -71,20 +214,20 @@ class Symbol:
     def __getattr__(self, name):
         if name.startswith('_'):
             raise AttributeError(name) # ipython requirement for repr_html
-        if name == "df":
+        if name == "dataframe":
             if name in self._repo:
                 if self._repo[name] is None:
-                    self._repo[name] = self.array.to_dataframe()
+                    self._repo[name] = self.array.to_pandas()
                     return self._repo[name].copy()
                 else:
                     #Sanity check
                     df = self._repo[name]
-                    dense = df.values.reshape(self.array.shape)
-                    if np.allclose(dense, self.array.dense()):
+                    dense = df.value.values.reshape(self.array.shape)
+                    if np.allclose(dense, self.array.dense):
                         return df.copy()
                     else:
-                        print(f"Getting df again...")
-                        self._repo[name] = self.array.to_dataframe()
+                        print(f"Getting dataframe again...")
+                        self._repo[name] = self.array.to_pandas()
                         return self._repo[name].copy()
             else:
                 self.definition_msg(name) # dev msg
@@ -116,7 +259,7 @@ class Symbol:
         assert key in symbol_handler.symbols_book, f"'{key}' is not present in 'symbol_handler.symbols_book' dictionary"
 
         file_path = symbol_handler.get_info(*key)
-        symbol_dict = from_feather_dict(file_path)
+        symbol_dict = from_feather_dict(file_path, with_=ka.settings.feather_with)
         self.name=                 name
         self.value_type=           value_type
         self.metadata=             symbol_dict['metadata']
@@ -162,20 +305,30 @@ class Symbol:
     def dims(self):
         return self.array.dims
 
+    @property
+    def df(self):
+        return self.dataframe.set_index(self.array.dims)
+
     @property
     def dfm(self):
-        dfm = self.df.reset_index()
+        dfm = self.dataframe
         for k, v in self.metadata.items():
             dfm[k] = dfm["id"].map(v)
         return dfm
 
     @property
     def dfc(self):
-        dfc = self.df.reset_index()
+        dfc = self.dataframe
         for k, v in self.metadata.items():
             if 'custom_' in k:
                 dfc[k] = dfc["id"].map(v)
         return dfc
+    
+    def to_polars(self):
+        return self.array.to_polars()
+
+    def to_pandas(self):
+        return self.array.to_pandas()
 
     def metadata_union(self, other=None):
         self_metadata = self.metadata
@@ -300,9 +453,9 @@ class Symbol:
     def rename(self, new_name: str):
         return self.new_symbol(self.array, new_name)
 
-    def dimreduc(self, dim:str='h', aggfunc:str='sum'):
+    def dimreduc(self, dim:str='h', aggfunc=np.add.reduce):
         '''
-        aggfunc in ['sum','mul','mean'], defult 'sum'
+        aggfunc in [np.add.reduce,np.multiply.reduce,np.average]. defult np.add.reduce
         '''
         new_array = self.array.reduce(dim, aggfunc)
         new_name = f"({self.name}).dimreduc({dim},{aggfunc})"
@@ -312,7 +465,6 @@ class Symbol:
     def items(self):
         return self.array.coords
 
-
     def rename_dim(self, **kwargs):
         """ This function renames a dimension in a symbol.
 
@@ -346,105 +498,36 @@ class Symbol:
         else:
             raise Exception('value is neither str nor dict')
 
-    def elems2date(self, dim:str=None, reference_date:str='01-01-2030', freq:str='H'):
-        new_array = self.array.elems2datetime(dim=dim, reference_date=reference_date, freq=freq)
-        new_name = f"({self.name}).elems2date({dim},{reference_date},{freq})"
+    def dropna(self):
+        new_array = self.array.dropna()
+        new_name = f"({self.name}).dropna()"
         return self.new_symbol(new_array, new_name)
 
-    def dropna(self, infinite=True):
-        new_array = self.array.dropna(infinite)
-        new_name = f"({self.name}).dropna({infinite=})"
+    def dropinf(self, pos, neg):
+        new_array = self.array.dropinf(pos,neg)
+        new_name = f"({self.name}).dropna({pos=},{neg=})"
         return self.new_symbol(new_array, new_name)
 
-#     def round(self, decimals:int):
-#         df = self.df.set_index(['id'] + self.get("dims")).drop("symbol", axis=1)
-#         df['value'] = df['value'].round(decimals)
-#         return self.new_symbol(self, df, f"{self.name}.round({str(decimals)})", self.get("dims"))
+    def drop(self, dims:Union[str,List[str]]):
+        new_array = self.array.drop(dims=dims)
+        new_name = f"({self.name}).dropna({dims=})"
+        return self.new_symbol(new_array, new_name)
 
-    # def elems2str(self, by='h', string='t', digits=4):
-    #     df = self.df.copy()
-    #     df[by] = df[by].apply(lambda x: string+str(x).zfill(digits))
-    #     new_object = self*1
-    #     new_object.dims = self.get('dims')
-    #     new_object.df = df
-    #     new_object.info = self.info
-    #     new_object.name = f"({self.get('name')}).elems2str({by},{string},{str(digits)})"
-    #     return new_object
+    def round(self, decimals:int):
+        new_array = self.array.round(decimals=decimals)
+        new_name = f"({self.name}).round({decimals=})"
+        return self.new_symbol(new_array, new_name)
 
-    # def elems2int(self, by='h'):
-    #     df = self.df.copy()
-    #     df.loc[:, by] = df[by].str.extract(r"(\d+)", expand=False).astype("int16")
-    #     new_object = self*1
-    #     new_object.dims = self.get('dims')
-    #     new_object.df = df
-    #     new_object.info = self.info
-    #     new_object.name = f"({self.get('name')}).elems2int({by})"
-    #     return new_object
+    def elems_to_datetime(self, new_dim:str, actual_dim:str, reference_date:str='01-01-2030', freq:str='H', sort_corrds:bool=True):
+        new_array = self.array.elems_to_datetime(new_dim=new_dim, actual_dim=actual_dim, reference_date=reference_date, freq=freq, sort_coords=sort_corrds)
+        new_name = f"({self.name}).elems_to_datetime({new_dim}{actual_dim},{reference_date},{freq},{sort_corrds})"
+        return self.new_symbol(new_array, new_name)
 
-#     def replacezero(self, by=1):
-#         df = self.df.copy()
-#         df['value'] = df['value'].replace(0.0,by)
-#         new_object = self*1
-#         new_object.dims = self.get('dims')
-#         new_object.df = df
-#         new_object.info = self.info
-#         new_object.name = f"({self.get('name')}).replacezero(by={str(by)})"
-#         return new_object
+    def elems_to_int(self, new_dim:str, actual_dim:str):
+        new_array = self.array.elems_to_int(new_dim, actual_dim)
+        new_name = f"({self.name}).elems_to_int({new_dim=},{actual_dim=})"
+        return self.new_symbol(new_array, new_name)
 
-#     def replaceall(self, by=1):
-#         df = self.df.copy()
-#         df['value'] = by
-#         new_object = self*1
-#         new_object.dims = self.get('dims')
-#         new_object.df = df
-#         new_object.info = self.info
-#         new_object.name = f"({self.get('name')}).replaceall(by={str(by)})"
-#         return new_object
-
-#     def replacenan(self, by=0):
-#         df = self.df.copy()
-#         df['value'] = df['value'].fillna(by)
-#         new_object = self*1
-#         new_object.dims = self.get('dims')
-#         new_object.df = df
-#         new_object.info = self.info
-#         new_object.name = f"({self.get('name')}).replacenan(by={str(by)})"
-
-#         return new_object
-
-#     def fillelems(self):
-#         dims = self.dims
-#         if len(dims) == 0:
-#             raise Exception("Symbol without dimensions")
-#         elif len(dims) == 1:
-#             df = pd.concat({'value':
-#                                 self.df.drop("symbol", axis=1)
-#                                 .pivot_table(
-#                                             index=['id'],
-#                                             columns=dims,
-#                                             values='value',
-#                                             aggfunc=sum,
-#                                             )
-#                                 .fillna(0)
-#                                 .sort_index()
-#                             },
-#                             names=[''], axis=1,
-#                             ).stack(dims)
-#         elif len(dims) > 1:
-#             if 'h' in dims:
-#                 dim = 'h'
-#             else:
-#                 dim = dims[0]
-#             common_dims = [elem for elem in dims if elem != dim]
-#             df = self.reorganize(dim, common_dims)
-#             df = df.stack(common_dims)
-#         new_object = self*1
-#         new_object.dims = self.get('dims')
-#         new_object.df = df.reset_index()
-#         new_object.info = self.info
-#         new_object.name = f"({self.get('name')}).fillelems()"
-#         return new_object
-    
     def find_ids(self, **kwargs):
         '''
         find ids whose headings comply the criteria to the value indicated
@@ -650,55 +733,6 @@ class Symbol:
     def __repr__(self):
         return f'''Symbol(name='{self.name}', \n       value_type='{self.value_type}')'''
 
-#     def transform(self, subset_of_sets=['n'], func='sum', condition='!=', value=0):
-#         '''
-#         transform consists of providing a list of sets to group the dataframe and apply the function. Then the result is compared with a condition and a value. If the condition is true, the resulting rows are kept, otherwise it is dropped. 
-#         subset_of_sets is a list of sets that are present in the symbol.
-#         At least one set must be left out of subset_of_sets to apply the function.
-        
-#         eg: N_TECH.transform(subset_of_sets=['n'], func='sum', condition='!=', value=0)
-#             As N_TECH has 'n' and 'tech' as sets, the function is applied to 'n' and agregating all 'tech' and the result is compared with '!= 0'.
-#             The final result is a dataframe without elements of 'n' that has a sum of element of 'tech' equal to zero.
-#             It is a way to clean up the dataframe by removing elements of a set that are not needed.
-#         '''
-        
-#         ops = {'>': operator.gt,
-#                 '<': operator.lt,
-#                 '>=': operator.ge,
-#                 '<=': operator.le,
-#                 '==': operator.eq,
-#                 '!=': operator.ne}
-        
-#         keep = ops[condition](self.df.groupby(["id"]+subset_of_sets)["value"].transform(func), value)
-#         df = self.df.loc[keep]
-#         new_object = self*1
-#         new_object.dims = self.get('dims')
-#         new_object.df = df
-#         new_object.info = self.info
-#         new_object.name = f"({self.get('name')}).transform({subset_of_sets},{func},{condition},{value})"
-#         return new_object
 
-
-
-
-
-# def symbols_operation_on_gpu(symbol_operation):     
-#     @wraps(symbol_operation)
-#     def wrapper(cls, other):
-#         if settings.GPU:
-#             if isinstance(cls, Symbol) and isinstance(other, Symbol):
-#                 with cupy.cuda.Device(0):
-#                     logger.info('Using GPU')
-#                     cls.array = cls.array.astype(float).cupy.as_cupy()
-#                     other.array = other.array.astype(float).cupy.as_cupy()
-#                     new_cls = symbol_operation(cls, other)
-#                     new_cls.array = new_cls.array.astype(float).cupy.as_numpy()
-#                     cls.array = cls.array.astype(float).cupy.as_numpy()
-#                     other.array = other.array.astype(float).cupy.as_numpy()
-#                 logger.info('Finished GPU')
-#                 return new_cls
-#             else:
-#                 return symbol_operation(cls, other)
-#         else:
-#             return symbol_operation(cls, other)
-#     return wrapper
\ No newline at end of file
+def from_feather(path):
+    return Symbol(**from_feather_dict(path, with_=ka.settings.feather_with))
\ No newline at end of file
