@@ -10,7 +10,7 @@ syx.compress_subdirs(folder=folder, zip_extension='7z', delete=False)
 DC = DataCollection()
 DC.add_collector(collector_name='csv_collector', parser=syx.symbol_parser_csv, loader=syx.load_csv)
 DC.add_folder('csv_collector', folder)
-DC.add_custom_attr(collector_name='csv_collector', keep_zeros=False)
+DC.add_custom_attr(collector_name='csv_collector')
 DC.adquire(id_integer=False, zip_extension='7z')
 # DC.adquire(id_integer=False, zip_extension=None)
 SH = SymbolsHandler(method='object', obj=DC)
@@ -33,7 +33,7 @@ syx.compress_subdirs(folder=folder, zip_extension='7z', delete=False)
 DC = DataCollection()
 DC.add_collector(collector_name='csv_collector', parser=syx.symbol_parser_csv, loader=syx.load_csv)
 DC.add_folder('csv_collector', folder)
-DC.add_custom_attr(collector_name='csv_collector', keep_zeros=False)
+DC.add_custom_attr(collector_name='csv_collector')
 # DC.adquire(id_integer=False, zip_extension='7z')
 DC.adquire(id_integer=False, zip_extension=None)
 SH = SymbolsHandler(method='object', obj=DC)
@@ -52,7 +52,7 @@ syx.compress_subdirs(folder=folder, zip_extension='7z', delete=False)
 DC = DataCollection()
 DC.add_collector(collector_name='feather_collector', parser=syx.symbol_parser_feather, loader=syx.load_feather)
 DC.add_folder('feather_collector', folder)
-DC.add_custom_attr(collector_name='feather_collector', keep_zeros=False)
+DC.add_custom_attr(collector_name='feather_collector')
 # DC.adquire(id_integer=False, zip_extension='7z')
 DC.adquire(id_integer=False, zip_extension=None)
 SH = SymbolsHandler(method='object', obj=DC)
@@ -74,8 +74,7 @@ DC.add_collector(collector_name='gdx_collector', parser=syx.symbol_parser_gdx, l
 DC.add_folder('gdx_collector', folder)
 DC.add_custom_attr(collector_name='gdx_collector', 
                     inf_to_zero=True, 
-                    verbose=False, 
-                    keep_zeros=False)
+                    verbose=False)
 # DC.adquire(id_integer=True, gams_dir=os.getenv("GAMS_DIR"), zip_extension='7z')
 DC.adquire(id_integer=False, gams_dir=os.getenv("GAMS_DIR"), zip_extension=None)
 SH = SymbolsHandler(method='object', obj=DC)
